@@ -203,6 +203,12 @@ static int bitmap_loadfont_locked(const char *filename)
 						else
 							memcpy(font[i], conio_fontdata[current_font[i]].eight_by_sixteen, fontsize);
 						break;
+					default:
+						goto error_return;
+				}
+				break;
+			case 12:
+				switch (vstat.charheight) {
 					case 20:
 						if(conio_fontdata[current_font[i]].twelve_by_twenty==NULL) {
 							if (i==0)
@@ -266,20 +272,20 @@ bitmap_vmem_puttext_locked(int sx, int sy, int ex, int ey, struct vmem_cell *fil
 	for(y=sy-1;y<ey;y++) {
 		vc = vmem_cell_ptr(vstat.vmem, sx - 1, y);
 		for(x=sx-1;x<ex;x++) {
-			if (vstat.mode == PRESTEL_40X25 && ((vc->bg & 0x02000000) || (fi->bg & 0x02000000))) {
-				if ((vc->bg & 0x01000000) != (fi->bg & 0x01000000)) {
+			if (vstat.mode == PRESTEL_40X25 && ((vc->bg & CIOLIB_BG_PRESTEL) || (fi->bg & CIOLIB_BG_PRESTEL))) {
+				if ((vc->bg & CIOLIB_BG_DOUBLE_HEIGHT) != (fi->bg & CIOLIB_BG_DOUBLE_HEIGHT)) {
 					// *ANY* change to double-height potentially changes
 					// *EVERY* character on the screen
 					fullredraw = true;
 				}
 			}
 			*vc = *(fi++);
-			if (vstat.mode == PRESTEL_40X25 && (vc->bg & 0x02000000)) {
+			if (vstat.mode == PRESTEL_40X25 && (vc->bg & CIOLIB_BG_PRESTEL)) {
 				if (vc->legacy_attr & 0x08) {
 					if (cio_api.options & CONIO_OPT_PRESTEL_REVEAL)
-						vc->bg |= 0x08000000;
+						vc->bg |= CIOLIB_BG_REVEAL;
 					else
-						vc->bg &= ~0x08000000;
+						vc->bg &= ~CIOLIB_BG_REVEAL;
 				}
 			}
 			vc = vmem_next_ptr(vstat.vmem, vc);
@@ -314,25 +320,25 @@ set_vmem_cell(size_t x, size_t y, uint16_t cell, uint32_t fg, uint32_t bg)
 	vc->legacy_attr = cell >> 8;
 	vc->ch = cell & 0xff;
 	vc->fg = fg;
-	if (vstat.mode == PRESTEL_40X25 && ((vc->bg & 0x02000000) || (bg & 0x02000000))) {
+	if (vstat.mode == PRESTEL_40X25 && ((vc->bg & CIOLIB_BG_PRESTEL) || (bg & CIOLIB_BG_PRESTEL))) {
 		if (vc->legacy_attr & 0x08) {
 			if (cio_api.options & CONIO_OPT_PRESTEL_REVEAL)
-				vc->bg |= 0x08000000;
+				vc->bg |= CIOLIB_BG_REVEAL;
 			else
-				vc->bg &= ~0x08000000;
+				vc->bg &= ~CIOLIB_BG_REVEAL;
 		}
-		if ((vc->bg & 0x01000000) != (bg & 0x01000000)) {
+		if ((vc->bg & CIOLIB_BG_DOUBLE_HEIGHT) != (bg & CIOLIB_BG_DOUBLE_HEIGHT)) {
 			// *ANY* change to double-height potentially changes
 			// *EVERY* character on the screen
 			request_redraw_locked();
 		}
 	}
-	if (vstat.mode == PRESTEL_40X25 && (vc->bg & 0x02000000)) {
+	if (vstat.mode == PRESTEL_40X25 && (vc->bg & CIOLIB_BG_PRESTEL)) {
 		if (vc->legacy_attr & 0x08) {
 			if (cio_api.options & CONIO_OPT_PRESTEL_REVEAL)
-				vc->bg |= 0x08000000;
+				vc->bg |= CIOLIB_BG_REVEAL;
 			else
-				vc->bg &= ~0x08000000;
+				vc->bg &= ~CIOLIB_BG_REVEAL;
 		}
 	}
 	vc->bg = bg;
@@ -504,7 +510,7 @@ static struct rectlist *alloc_full_rect(struct bitmap_screen *screen, bool allow
 
 static uint32_t color_value(uint32_t col)
 {
-	if (col & 0x80000000)
+	if (col & CIOLIB_COLOR_RGB)
 		return col & 0xffffff;
 	if ((col & 0xffffff) < sizeof(palette) / sizeof(palette[0]))
 		return palette[col & 0xffffff] & 0xffffff;
@@ -563,9 +569,10 @@ struct charstate {
 	uint32_t fontoffset;
 	bool slow;
 	bool gexpand;
-	int8_t extra_rows;
+	bool did_top;
 	bool double_height;
 	bool sep;
+	bool top_half;
 };
 
 struct blockstate {
@@ -579,7 +586,7 @@ struct blockstate {
 static bool
 can_cheat(struct blockstate *bs, struct vmem_cell *vc)
 {
-	return vc->bg == bs->cheat_colour && (vc->ch == ' ') && (vc->font < CONIO_FIRST_FREE_FONT) && !(vc->bg & 0x02000000);
+	return vc->bg == bs->cheat_colour && (vc->ch == ' ') && (vc->font < CONIO_FIRST_FREE_FONT) && !(vc->bg & CIOLIB_BG_PRESTEL);
 }
 
 static void
@@ -588,9 +595,10 @@ calc_charstate(struct blockstate *bs, struct vmem_cell *vc, struct charstate *cs
 	bool not_hidden = true;
 	cs->slow = bs->font_data_width != 8;
 	cs->bg = vc->bg;
-	cs->extra_rows = 0;
+	cs->did_top = false;
 	cs->sep = false;
 	cs->double_height = false;
+	cs->top_half = true;
 
 	if (vstat.forced_font) {
 		cs->font = vstat.forced_font;
@@ -633,13 +641,13 @@ calc_charstate(struct blockstate *bs, struct vmem_cell *vc, struct charstate *cs
 	}
 	uint32_t fg = vc->fg;
 
-	if (vstat.mode == PRESTEL_40X25 && (vc->bg & 0x02000000)) {
-		unsigned char lattr;
+	if (vstat.mode == PRESTEL_40X25 && (vc->bg & CIOLIB_BG_PRESTEL)) {
 		bool top = false;
 		bool bottom = false;
+		unsigned char lattr;
 
 		cs->slow = true;
-		if (vc->bg & 0x20000000 && vc->ch >= 160)
+		if (vc->bg & CIOLIB_BG_SEPARATED && vc->ch >= 160)
 			cs->sep = true;
 		// Start at the first cell...
 		struct vmem_cell *pvc = vmem_cell_ptr(vstat.vmem, 0, 0);
@@ -658,7 +666,7 @@ calc_charstate(struct blockstate *bs, struct vmem_cell *vc, struct charstate *cs
 				pvc = vmem_cell_ptr(vstat.vmem, 0, y);
 				for (int x = 0; x < vstat.cols; x++) {
 					// If there's at least one top, this is a top row
-					if (pvc->bg & 0x01000000) {
+					if (pvc->bg & CIOLIB_BG_DOUBLE_HEIGHT) {
 						top = true;
 						break;
 					}
@@ -673,38 +681,52 @@ calc_charstate(struct blockstate *bs, struct vmem_cell *vc, struct charstate *cs
 
 		// If this is a bottom row...
 		if (bottom) {
+			/*
+			 * TODO: With Commstar, it copies to the next
+			 * row when it's initially written, so if you
+			 * overwrite the double height on the first row,
+			 * the second row retains the double height and
+			 * becomes tops.
+			 *
+			 * This violates the Prestel Terminal Specification,
+			 * so for now I'm doing this the specified way
+			 * rather than emulating the Commstar bug because
+			 * it would be too hard to implement in this
+			 * implementation.  For v2 however, this should
+			 * likely be fixed to have the Commstar bug since
+			 * I expect that will be how the Prestel terminals
+			 * that use the SAA5050 (ie: most of them) do it.
+			 */
+			if (vc->bg & CIOLIB_BG_PRESTEL_TERMINAL) {
+				pvc = vmem_prev_row_ptr(vstat.vmem, vc);
+				cs->bg = pvc->bg;
+				fg = pvc->fg;
+				if (pvc->bg & CIOLIB_BG_SEPARATED && pvc->ch >= 160)
+					cs->sep = true;
+			}
+			else
+				pvc = vc;
 			assert(!top);
-			// Find the cell above this one...
-			pvc = vmem_prev_row_ptr(vstat.vmem, vc);
-			if (pvc->bg & 0x01000000) {
-				// If it's double-height, draw the cell above
+			if (pvc->bg & CIOLIB_BG_DOUBLE_HEIGHT) {
 				cs->double_height = true;
-				cs->extra_rows = -(vstat.charheight);
-				cs->fontoffset = (pvc->ch) * (vstat.charheight * ((bs->font_data_width + 7) / 8));
-				if (pvc->ch >= 160) {
-					if (pvc->bg & 0x20000000)
-						cs->sep = true;
-					else
-						cs->sep = false;
-				}
-				else
-					cs->sep = false;
-				// TODO: Update FS etc.
+				cs->fontoffset = (pvc->ch) * (vstat.charheight * ((bs->font_data_width + 7) / 8)) + ((vstat.charheight * ((bs->font_data_width + 7) / 8)) / 2);
 			}
 			else {
 				// Draw as space if not double-bottom
 				cs->fontoffset=(32) * (vstat.charheight * ((bs->font_data_width + 7) / 8));
 			}
-			fg = pvc->fg;
-			cs->bg = pvc->bg;
+			cs->top_half = false;
 			lattr = pvc->legacy_attr;
+			// Handle blink
+			if (vc->bg & CIOLIB_BG_PRESTEL_TERMINAL)
+				draw_fg = ((!(lattr & 0x80)) || vstat.no_blink);
 		}
 		// If not a bottom (either a top or no doubles)
 		else {
 			// And it's a top row...
 			if (top) {
 				// And it's double-height
-				if (vc->bg & 0x01000000) {
+				if (vc->bg & CIOLIB_BG_DOUBLE_HEIGHT) {
 					//assert(top);
 					// Draw it as is where is
 					cs->double_height = true;
@@ -758,22 +780,13 @@ draw_char_row_slow(struct blockstate *bs, struct charstate *cs, uint32_t y)
 {
 	bool fbb;
 	int pixeloffset;
-	int pixeloffset2;
 
+	pixeloffset = bs->pixeloffset;
 	if (cs->double_height) {
-		pixeloffset = bs->pixeloffset + cs->extra_rows * screena.screenwidth;
-		if (pixeloffset >= bs->maxpix)
-			pixeloffset -= bs->maxpix;
-		if (pixeloffset < 0)
-			pixeloffset += bs->maxpix;
-		pixeloffset2 = bs->pixeloffset + (cs->extra_rows + 1) * screena.screenwidth;
-		if (pixeloffset2 < 0)
-			pixeloffset2 += bs->maxpix;
-		if (pixeloffset2 >= bs->maxpix)
-			pixeloffset2 -= bs->maxpix;
+		y /= 2;
+		if (!cs->top_half)
+			y += vstat.charheight / 2;
 	}
-	else
-		pixeloffset = bs->pixeloffset;
 
 	uint8_t fb = cs->font[cs->fontoffset];
 	for(unsigned x = 0; x < vstat.charwidth; x++) {
@@ -823,27 +836,15 @@ draw_char_row_slow(struct blockstate *bs, struct charstate *cs, uint32_t y)
 		}
 		pixeloffset++;
 		assert(pixeloffset < bs->maxpix || x == (vstat.charwidth - 1));
-		if (cs->double_height) {
-			if (screena.rect->data[pixeloffset2] != ac) {
-				screena.rect->data[pixeloffset2] = ac;
-				screena.update_pixels = 1;
-			}
-			if (screenb.rect->data[pixeloffset2] != bc) {
-				screenb.rect->data[pixeloffset2] = bc;
-				screenb.update_pixels = 1;
-			}
-			pixeloffset2++;
-			assert(pixeloffset2 < bs->maxpix || x == (vstat.charwidth - 1));
-		}
 	}
 	if (cs->double_height) {
-		cs->extra_rows++;
-		bs->pixeloffset += vstat.charwidth;
-		assert(bs->pixeloffset <= bs->maxpix);
-	}
-	else {
-		bs->pixeloffset = pixeloffset;
+		// Go back for next row
+		if (!cs->did_top) {
+			cs->fontoffset -= ((bs->font_data_width + 7) / 8);
+		}
+		cs->did_top = !cs->did_top;
 	}
+	bs->pixeloffset = pixeloffset;
 }
 
 static void
@@ -919,7 +920,8 @@ bitmap_draw_vmem_locked(int sx, int sy, int ex, int ey, struct vmem_cell *fill)
 		for (unsigned vy = 0; vy < vheight; vy++) {
 			int coff = vmem_cell_offset(vstat.vmem, sx - 1, sy - 1 + vy);
 			for (unsigned vx = 0; vx < vwidth; vx++) {
-				bitmap_drawn[coff] = fill[foff++];
+				if (bitmap_drawn)
+					bitmap_drawn[coff] = fill[foff++];
 				coff = vmem_next_offset(vstat.vmem, coff);
 			}
 		}
@@ -931,7 +933,8 @@ bitmap_draw_vmem_locked(int sx, int sy, int ex, int ey, struct vmem_cell *fill)
 			// Fill in charstate for this pass
 			int coff = vmem_cell_offset(vstat.vmem, sx - 1, sy - 1 + vy);
 			for (unsigned vx = 0; vx < vwidth; vx++) {
-				bitmap_drawn[coff] = fill[foff++];
+				if (bitmap_drawn)
+					bitmap_drawn[coff] = fill[foff++];
 				coff = vmem_next_offset(vstat.vmem, coff);
 				calc_charstate(&bs, &fill[vy * vwidth + vx], &charstate[vx], sx + vx, sy + vy);
 				if (charstate[vx].slow == false)
@@ -1181,15 +1184,15 @@ same_cell(struct vmem_cell *bitmap_cell, struct vmem_cell *c2)
 	if (bitmap_cell->ch != c2->ch)
 		return false;
 	// Handles reveal/unreveal updates, modifies vmem
-	if (vstat.mode == PRESTEL_40X25 && (c2->bg & 0x02000000)) {
+	if (vstat.mode == PRESTEL_40X25 && (c2->bg & CIOLIB_BG_PRESTEL)) {
 		if (c2->legacy_attr & 0x08) {
 			if (cio_api.options & CONIO_OPT_PRESTEL_REVEAL)
-				c2->bg |= 0x08000000;
+				c2->bg |= CIOLIB_BG_REVEAL;
 			else
-				c2->bg &= ~0x08000000;
+				c2->bg &= ~CIOLIB_BG_REVEAL;
 		}
 	}
-	if (bitmap_cell->bg & 0x10000000)	// Dirty.
+	if (bitmap_cell->bg & CIOLIB_BG_DIRTY)	// Dirty.
 		return false;
 	if (bitmap_cell->bg != c2->bg)
 		return false;
@@ -1307,7 +1310,7 @@ static int update_from_vmem(int force)
 	for(y=0;y<height;y++) {
 		for(x=0;x<width;x++) {
 			/* Has this char been updated? */
-			if(force || !same_cell(&bitmap_drawn[pos], &vstat.vmem->vmem[pos])
+			if(force || bitmap_drawn == NULL || !same_cell(&bitmap_drawn[pos], &vstat.vmem->vmem[pos])
 			    || ((vstat.vmem->vmem[pos].legacy_attr & 0x80)
 				&& blink_attr_changed)
 			    || ((vstat.vmem->vmem[pos].legacy_attr & 0x08) && bright_attr_changed))
@@ -1389,12 +1392,11 @@ bitmap_vmem_puttext(int sx, int sy, int ex, int ey, struct vmem_cell *fill)
 	return ret;
 }
 
-int bitmap_vmem_gettext(int sx, int sy, int ex, int ey, struct vmem_cell *fill)
+static bool
+bitmap_vmem_gettext_invalid(int sx, int sy, int ex, int ey, struct vmem_cell *fill)
 {
-	int x,y;
-
 	if(!bitmap_initialized)
-		return(0);
+		return true;
 
 	if(		   sx < 1
 			|| sy < 1
@@ -1405,10 +1407,16 @@ int bitmap_vmem_gettext(int sx, int sy, int ex, int ey, struct vmem_cell *fill)
 			|| ex > cio_textinfo.screenwidth
 			|| ey > cio_textinfo.screenheight
 			|| fill==NULL) {
-		return(0);
+		return true;
 	}
+	return false;
+}
+
+static int
+bitmap_vmem_gettext_locked_inner(int sx, int sy, int ex, int ey, struct vmem_cell *fill)
+{
+	int x,y;
 
-	assert_rwlock_rdlock(&vstatlock);
 	for(y=sy-1;y<ey;y++) {
 		struct vmem_cell *vc = vmem_cell_ptr(vstat.vmem, sx - 1, y);
 		for(x=sx-1;x<ex;x++) {
@@ -1416,10 +1424,29 @@ int bitmap_vmem_gettext(int sx, int sy, int ex, int ey, struct vmem_cell *fill)
 			vc = vmem_next_ptr(vstat.vmem, vc);
 		}
 	}
-	assert_rwlock_unlock(&vstatlock);
 	return(1);
 }
 
+static int
+bitmap_vmem_gettext_locked(int sx, int sy, int ex, int ey, struct vmem_cell *fill)
+{
+	if (bitmap_vmem_gettext_invalid(sx, sy, ex, ey, fill))
+		return 0;
+
+	return bitmap_vmem_gettext_locked_inner(sx, sy, ex, ey, fill);
+}
+
+int bitmap_vmem_gettext(int sx, int sy, int ex, int ey, struct vmem_cell *fill)
+{
+	if (bitmap_vmem_gettext_invalid(sx, sy, ex, ey, fill))
+		return 0;
+
+	assert_rwlock_rdlock(&vstatlock);
+	int ret = bitmap_vmem_gettext_locked_inner(sx, sy, ex, ey, fill);
+	assert_rwlock_unlock(&vstatlock);
+	return(ret);
+}
+
 void bitmap_gotoxy(int x, int y)
 {
 	if(!bitmap_initialized)
@@ -1547,11 +1574,20 @@ int bitmap_setfont(int font, int force, int font_num)
 		ow=ti.screenwidth;
 		oh=ti.screenheight;
 
+		// Tell coverity this assignment is not protected by the lock
+		// coverity[def:SUPPRESS]
 		old=malloc(ow*oh*sizeof(*old));
 		if(old) {
-			bitmap_vmem_gettext(1,1,ow,oh,old);
-			/* coverity[sleep:SUPPRESS] */
+			/*
+			 * Passing old to bitmap_vmem_gettext_locked()
+			 * gets Coverity worked up about old being accessed
+			 * after an unlock/lock, so we have the suppressions
+			 * below.
+			 */
+			bitmap_vmem_gettext_locked(1,1,ow,oh,old);
+			assert_rwlock_unlock(&vstatlock);
 			textmode(newmode);
+			assert_rwlock_wrlock(&vstatlock);
 			new=malloc(ti.screenwidth*ti.screenheight*sizeof(*new));
 			if(!new) {
 				free(old);
@@ -1665,7 +1701,8 @@ bitmap_movetext_screen(int x, int y, int tox, int toy, int direction, int height
 		// Fill the bits with impossible data so they're redrawn
 		int bdoff = vmem_cell_offset(vstat.vmem, 0, toy - 1);
 		for (int vy = 0; vy < height; vy++) {
-			memset(&bitmap_drawn[bdoff], 0x04, sizeof(*bitmap_drawn) * vstat.cols);
+			if (bitmap_drawn)
+				memset(&bitmap_drawn[bdoff], 0x04, sizeof(*bitmap_drawn) * vstat.cols);
 			bdoff = vmem_next_row_offset(vstat.vmem, bdoff);
 		}
 		if (vstat.charheight * vstat.rows == screena.screenheight) {
@@ -1760,9 +1797,9 @@ int bitmap_movetext(int x, int y, int ex, int ey, int tox, int toy)
 
 		// Set up the move back down...
 		scrolldown = !scrolldown;
-		height = vstat.rows - height;
+		height = vstat.rows - height - 1;
 		toy = vstat.rows - (height - 1);
-		y = toy - height + 1;
+		y = vstat.rows - height;
 	}
 	if (scrolldown) {
 		soff = vmem_cell_offset(vstat.vmem, x - 1, y + height - 2);
@@ -1774,7 +1811,8 @@ int bitmap_movetext(int x, int y, int ex, int ey, int tox, int toy)
 	}
 	for(cy=0; cy<height; cy++) {
 		memmove(&vstat.vmem->vmem[doff], &vstat.vmem->vmem[soff], sizeof(vstat.vmem->vmem[0])*width);
-		memmove(&bitmap_drawn[doff], &bitmap_drawn[soff], sizeof(vstat.vmem->vmem[0])*width);
+		if (bitmap_drawn)
+			memmove(&bitmap_drawn[doff], &bitmap_drawn[soff], sizeof(vstat.vmem->vmem[0])*width);
 		if (scrolldown) {
 			soff = vmem_prev_row_offset(vstat.vmem, soff);
 			doff = vmem_prev_row_offset(vstat.vmem, doff);
@@ -1785,7 +1823,13 @@ int bitmap_movetext(int x, int y, int ex, int ey, int tox, int toy)
 		}
 	}
 
-	bitmap_movetext_screen(x, oy, tox, otoy, oscrolldown ? -1 : 1, oheight, width);
+	// Make the whole thing redraw
+	if (vstat.mode == PRESTEL_40X25) {
+		if (bitmap_drawn)
+			memset(bitmap_drawn, 0x04, sizeof(struct vmem_cell) * vstat.cols * vstat.rows);
+	}
+	else
+		bitmap_movetext_screen(x, oy, tox, otoy, oscrolldown ? -1 : 1, oheight, width);
 	assert_rwlock_unlock(&vstatlock);
 
 	return(1);
@@ -1943,11 +1987,12 @@ int bitmap_setpixel(uint32_t x, uint32_t y, uint32_t colour)
 	}
 	if (xchar < vstat.cols && ychar < vstat.rows) {
 		int off = vmem_cell_offset(vstat.vmem, xchar, ychar);
-		if (!same_cell(&bitmap_drawn[off], &vstat.vmem->vmem[off])) {
+		if (bitmap_drawn == NULL || !same_cell(&bitmap_drawn[off], &vstat.vmem->vmem[off])) {
 			bitmap_draw_from_vmem(xchar + 1, ychar + 1, xchar + 1, ychar + 1, true);
 		}
-		vstat.vmem->vmem[off].bg |= 0x04000000;
-		bitmap_drawn[off].bg |= 0x04000000;
+		vstat.vmem->vmem[off].bg |= CIOLIB_BG_PIXEL_GRAPHICS;
+		if (bitmap_drawn)
+			bitmap_drawn[off].bg |= CIOLIB_BG_PIXEL_GRAPHICS;
 	}
 	if (x < screena.screenwidth && y < screena.screenheight) {
 		if (screena.rect->data[pixel_offset(&screena, x, y)] != colour) {
@@ -2031,14 +2076,14 @@ int bitmap_setpixels(uint32_t sx, uint32_t sy, uint32_t ex, uint32_t ey, uint32_
 				if (in_text_area) {
 					if (!yupdated) {
 						if (!xupdated) {
-							if (!same_cell(&bitmap_drawn[off], &vstat.vmem->vmem[off])) {
+							if (bitmap_drawn == NULL || !same_cell(&bitmap_drawn[off], &vstat.vmem->vmem[off])) {
 								bitmap_draw_from_vmem(charx + 1, chary + 1, charx + 1, chary + 1, true);
 							}
 							if (vstat.vmem && vstat.vmem->vmem) {
-								vstat.vmem->vmem[off].bg |= 0x04000000;
+								vstat.vmem->vmem[off].bg |= CIOLIB_BG_PIXEL_GRAPHICS;
 							}
 							if (bitmap_drawn) {
-								bitmap_drawn[off].bg |= 0x04000000;
+								bitmap_drawn[off].bg |= CIOLIB_BG_PIXEL_GRAPHICS;
 							}
 							xupdated = true;
 						}
@@ -2078,14 +2123,14 @@ int bitmap_setpixels(uint32_t sx, uint32_t sy, uint32_t ex, uint32_t ey, uint32_
 				if (in_text_area) {
 					if (!yupdated) {
 						if (!xupdated) {
-							if (!same_cell(&bitmap_drawn[off], &vstat.vmem->vmem[off])) {
+							if (bitmap_drawn == NULL || !same_cell(&bitmap_drawn[off], &vstat.vmem->vmem[off])) {
 								bitmap_draw_from_vmem(charx + 1, chary + 1, charx + 1, chary + 1, true);
 							}
 							if (vstat.vmem && vstat.vmem->vmem) {
-								vstat.vmem->vmem[off].bg |= 0x04000000;
+								vstat.vmem->vmem[off].bg |= CIOLIB_BG_PIXEL_GRAPHICS;
 							}
 							if (bitmap_drawn) {
-								bitmap_drawn[off].bg |= 0x04000000;
+								bitmap_drawn[off].bg |= CIOLIB_BG_PIXEL_GRAPHICS;
 							}
 							xupdated = true;
 						}
@@ -2453,6 +2498,122 @@ bitmap_snap(bool grow, int maxwidth, int maxheight)
 	} while ((vstat.winwidth > maxwidth || vstat.winheight > maxheight) && mult > 1);
 }
 
+static void
+integer_scale(int maxwidth, int maxheight, int64_t os)
+{
+	int64_t bs;
+	int64_t ls;
+	int64_t ns;
+	int w, h;
+
+	int mult = 1;
+	bitmap_get_scaled_win_size(mult, &w, &h, maxwidth, maxheight);
+	bs = ((int64_t)w * w) + ((int64_t)h * h);
+	ls = bs;
+	ns = bs;
+	while (ns < os) {
+		mult++;
+		bitmap_get_scaled_win_size(mult, &w, &h, 0, 0);
+		if ((maxwidth > 0) && (w > maxwidth)) {
+			mult--;
+			ns = ls;
+			break;
+		}
+		if (w == maxwidth)
+			break;
+		if ((maxheight > 0) && (h > maxheight)) {
+			mult--;
+			ns = ls;
+			break;
+		}
+		if (h == maxheight)
+			break;
+		bs = ((int64_t)w * w) + ((int64_t)h * h);
+		ls = ns;
+		ns = bs;
+	}
+	if ((os - ls) <= (ns - os)) {
+		if (mult > 1)
+			mult--;
+	}
+	bitmap_get_scaled_win_size(mult, &w, &h, maxwidth, maxheight);
+	vstat.winwidth = w;
+	vstat.winheight = h;
+	vstat.scaling = mult;
+}
+
+const double precision = 0.00005;
+const double tolerance = 0.0125;
+
+static void
+double_scale(int maxwidth, int maxheight, int64_t os)
+{
+	int64_t bs;
+	int64_t ls;
+	int64_t ns;
+	int w, h;
+
+	double mult = 1.0;
+	double step = 1.0;
+	bitmap_get_scaled_win_size(mult, &w, &h, maxwidth, maxheight);
+	bs = ((int64_t)w * w) + ((int64_t)h * h);
+	ls = bs;
+	ns = bs;
+	while (ns < os) {
+		mult += step;
+		bitmap_get_scaled_win_size(mult, &w, &h, 0, 0);
+		if ((maxwidth > 0) && (w > maxwidth)) {
+			mult -= step;
+			if (step <= precision) {
+				ns = ls;
+				break;
+			}
+			step /= 2;
+			continue;
+		}
+		if (w == maxwidth)
+			break;
+		if ((maxheight > 0) && (h > maxheight)) {
+			mult -= step;
+			if (step <= precision) {
+				ns = ls;
+				break;
+			}
+			step /= 2;
+			continue;
+		}
+		if (h == maxheight)
+			break;
+		bs = ((int64_t)w * w) + ((int64_t)h * h);
+		if (bs >= os && step > precision) {
+			mult -= step;
+			step /= 2;
+			continue;
+		}
+		ls = ns;
+		ns = bs;
+	}
+	long long lastDelta = llabs(os - ls);
+	long long currDelta = llabs(ns - os);
+	if (lastDelta <= currDelta) {
+		if (mult > 1) {
+			mult -= step;
+			currDelta = lastDelta;
+		}
+	}
+	bitmap_get_scaled_win_size(mult, &w, &h, maxwidth, maxheight);
+	vstat.winwidth = w;
+	vstat.winheight = h;
+	vstat.scaling = mult;
+}
+
+static double oldMult = 0.0;
+static double newMult = 0.0;
+static int oldWWidth = 0;
+static int oldWHeight = 0;
+static int oldSWidth = 0;
+static int oldSHeight = 0;
+
 /*
  * This function is intended to be called from the driver.
  * as a result, it cannot block waiting for driver status
@@ -2469,15 +2630,13 @@ int bitmap_drv_init_mode(int mode, int *width, int *height, int maxwidth, int ma
 {
 	int i;
 	int64_t os;
-	int64_t ls;
-	int64_t ns;
-	int64_t bs;
-	int w, h;
-	int mult;
 
 	if(!bitmap_initialized)
 		return(-1);
 
+	int prevSWidth = vstat.scrnwidth;
+	int prevSHeight = vstat.scrnheight;
+
 	if (mode == _ORIGMODE)
 		mode = C80;
 	if(load_vmode(&vstat, mode)) {
@@ -2530,40 +2689,31 @@ int bitmap_drv_init_mode(int mode, int *width, int *height, int maxwidth, int ma
 	cio_textinfo.winbottom=cio_textinfo.screenheight;
 
 	// Now calculate the closest diagonal new size that's smaller than max...
-	mult = 1;
-	bitmap_get_scaled_win_size(mult, &w, &h, maxwidth, maxheight);
-	bs = ((int64_t)w * w) + ((int64_t)h * h);
-	ls = bs;
-	ns = bs;
-	while (ns < os) {
-		mult++;
-		bitmap_get_scaled_win_size(mult, &w, &h, 0, 0);
-		if ((maxwidth > 0) && (w > maxwidth)) {
-			mult--;
-			ns = ls;
-			break;
+	double delta = fabs(vstat.scaling - round(vstat.scaling));
+	if (delta < tolerance)
+		integer_scale(maxwidth, maxheight, os);
+	else {
+		// If we're going back to the old scaling, and we haven't scaled, just restore the damn thing.
+		if (oldSWidth == vstat.scrnwidth && oldSHeight == vstat.scrnheight && vstat.scaling == newMult) {
+			vstat.winwidth = oldWWidth;
+			vstat.winheight = oldWHeight;
+			vstat.scaling = oldMult;
 		}
-		if (w == maxwidth)
-			break;
-		if ((maxheight > 0) && (h > maxheight)) {
-			mult--;
-			ns = ls;
-			break;
+		else {
+			int w, h;
+			// Tuck this away...
+			oldSWidth = prevSWidth;
+			oldSHeight = prevSHeight;
+			oldWWidth = vstat.winwidth;
+			oldWHeight = vstat.winheight;
+			oldMult = vstat.scaling;
+			double_scale(maxwidth, maxheight, os);
+			// Round-trip the scaling so it doesn't change randomly later...
+			bitmap_get_scaled_win_size(vstat.scaling, &w, &h, 0, 0);
+			vstat.scaling = bitmap_double_mult_inside(w, h);
 		}
-		if (h == maxheight)
-			break;
-		bs = ((int64_t)w * w) + ((int64_t)h * h);
-		ls = ns;
-		ns = bs;
+		newMult = vstat.scaling;
 	}
-	if ((os - ls) <= (ns - os)) {
-		if (mult > 1)
-			mult--;
-	}
-	bitmap_get_scaled_win_size(mult, &w, &h, maxwidth, maxheight);
-	vstat.winwidth = w;
-	vstat.winheight = h;
-	vstat.scaling = mult;
 
 	return(0);
 }
