@@ -82,6 +82,7 @@ struct sdl_keyvals {
 static pthread_mutex_t sdl_headlock;
 static struct rectlist *update_list = NULL;
 static struct rectlist *update_list_tail = NULL;
+extern int sdl_video_initialized;
 
 #ifdef __DARWIN__
 static int mac_width;
@@ -370,19 +371,10 @@ sdl_get_bounds(int *w, int *h)
 	return true;
 }
 
-static int sdl_init_mode(int mode, bool init)
+static int sdl_init_mode(int mode)
 {
 	int w, h;
 
-	if (mode != CIOLIB_MODE_CUSTOM) {
-		assert_rwlock_rdlock(&vstatlock);
-		if (mode == vstat.mode && !init) {
-			assert_rwlock_unlock(&vstatlock);
-			return 0;
-		}
-		assert_rwlock_unlock(&vstatlock);
-	}
-
 	sdl_user_func(SDL_USEREVENT_FLUSH);
 
 	assert_pthread_mutex_lock(&win_mutex);
@@ -434,7 +426,7 @@ int sdl_init(int mode)
 	_beginthread(sdl_video_event_thread, 0, NULL);
 #endif
 	sdl_user_func_ret(SDL_USEREVENT_INIT);
-	sdl_init_mode(ciolib_initial_mode, true);
+	sdl_init_mode(ciolib_initial_mode);
 
 	if(sdl_init_good) {
 		cio_api.mode=fullscreen?CIOLIB_MODE_SDL_FULLSCREEN:CIOLIB_MODE_SDL;
@@ -568,7 +560,7 @@ int sdl_getch(void)
 /* Called from main thread only */
 void sdl_textmode(int mode)
 {
-	sdl_init_mode(mode, false);
+	sdl_init_mode(mode);
 }
 
 /* Called from main thread only (Passes Event) */
@@ -1159,6 +1151,7 @@ void sdl_video_event_thread(void *data)
 					sdl_add_key(CIO_KEY_QUIT, &cvstat);
 				else {
 					sdl.QuitSubSystem(SDL_INIT_VIDEO);
+					sdl_video_initialized = FALSE;
 					return;
 				}
 				break;
@@ -1197,6 +1190,7 @@ void sdl_video_event_thread(void *data)
 					case SDL_USEREVENT_QUIT:
 						sdl_ufunc_retval=0;
 						sdl.QuitSubSystem(SDL_INIT_VIDEO);
+						sdl_video_initialized = FALSE;
 						sem_post(&sdl_ufunc_ret);
 						return;
 					case SDL_USEREVENT_FLUSH:
