@@ -282,24 +282,24 @@ static struct {
     {	0x4400, 0x5d00, 0x6700, 0x7100 }, /* key 68 - F10 */
     {	0xffff, 0xffff, 0xffff, 0xffff }, /* key 69 - num-lock - */
     {	0xffff, 0xffff, 0xffff, 0xffff }, /* key 70 - scroll-lock -  */
-    {	0x4700, 0x4737, 0x7700, 0xffff }, /* key 71 - home */
-    {	0x4800, 0x4838, 0x8d00, 0x9800 }, /* key 72 - cursor up */
-    {	0x4900, 0x4939, 0x8400, 0xffff }, /* key 73 - page up */
+    {	0x4700, 0x3700, 0x7700, 0xffff }, /* key 71 - home */
+    {	0x4800, 0x3800, 0x8d00, 0x9800 }, /* key 72 - cursor up */
+    {	0x4900, 0x3900, 0x8400, 0xffff }, /* key 73 - page up */
     {	0x4a2d, 0x4a2d, 0xffff, 0xffff }, /* key 74 - minus sign */
-    {	0x4b00, 0x4b34, 0x7300, 0xffff }, /* key 75 - cursor left */
+    {	0x4b00, 0x3400, 0x7300, 0xffff }, /* key 75 - cursor left */
     {	0xffff, 0x4c35, 0xffff, 0xffff }, /* key 76 - center key */
-    {	0x4d00, 0x4d36, 0x7400, 0xffff }, /* key 77 - cursor right */
+    {	0x4d00, 0x3600, 0x7400, 0xffff }, /* key 77 - cursor right */
     {	0x4e2b, 0x4e2b, 0xffff, 0xffff }, /* key 78 - plus sign */
-    {	0x4f00, 0x4f31, 0x7500, 0xffff }, /* key 79 - end */
-    {	0x5000, 0x5032, 0x9100, 0xa000 }, /* key 80 - cursor down */
-    {	0x5100, 0x5133, 0x7600, 0xffff }, /* key 81 - page down */
+    {	0x4f00, 0x3100, 0x7500, 0xffff }, /* key 79 - end */
+    {	0x5000, 0x3200, 0x9100, 0xa000 }, /* key 80 - cursor down */
+    {	0x5100, 0x3300, 0x7600, 0xffff }, /* key 81 - page down */
     {	CIO_KEY_IC, CIO_KEY_SHIFT_IC, CIO_KEY_CTRL_IC, CIO_KEY_ALT_IC}, /* key 82 - insert */
     {	CIO_KEY_DC, CIO_KEY_SHIFT_DC, CIO_KEY_CTRL_DC, CIO_KEY_ALT_IC}, /* key 83 - delete */
     {	0xffff, 0xffff, 0xffff, 0xffff }, /* key 84 - sys key */
     {	0xffff, 0xffff, 0xffff, 0xffff }, /* key 85 */
     {	0xffff, 0xffff, 0xffff, 0xffff }, /* key 86 */
-    {	0x8500, 0x5787, 0x8900, 0x8b00 }, /* key 87 - F11 */
-    {	0x8600, 0x5888, 0x8a00, 0x8c00 }, /* key 88 - F12 */
+    {	0x8500, 0x8700, 0x8900, 0x8b00 }, /* key 87 - F11 */
+    {	0x8600, 0x8800, 0x8a00, 0x8c00 }, /* key 88 - F12 */
 };
 
 static struct AtomDef *
