@@ -156,7 +156,9 @@ recv_nbytes(struct http_session *sess, uint8_t *buf, const size_t chunk_size, bo
 	while (received < chunk_size) {
 		ssize_t rc;
 		if (sess->is_tls) {
-#ifndef WITHOUT_CRYPTLIB
+#ifdef WITHOUT_CRYPTLIB
+			goto error_return;
+#else
 			int copied = 0;
 			int status = cryptPopData(sess->tls, &buf[received], chunk_size - received, &copied);
 			if (status == CRYPT_ERROR_COMPLETE) {
@@ -292,7 +294,9 @@ send_request(struct http_session *sess)
 	sess->cache.request_time = time(NULL);
 	ssize_t sent;
 	if (sess->is_tls) {
-#ifndef WITHOUT_CRYPTLIB
+#ifdef WITHOUT_CRYPTLIB
+		return false;
+#else
 		int copied;
 		int ret = cryptPushData(sess->tls, reqstr, len, &copied);
 		if (cryptStatusError(ret)) {
@@ -404,7 +408,7 @@ special_return:
 	return NULL;
 }
 
-const char *
+static const char *
 skipws(const char *val)
 {
 	while (*val == ' ' || *val == '\t')
@@ -743,15 +747,21 @@ parse_uri(struct http_session *sess)
 		goto error_return;
 	}
 	if (sess->req->uri[4] == 's') {
+#ifndef WITHOUT_CRYPTLIB
 		p = &sess->req->uri[5];
 		sess->is_tls = true;
 		sess->hacky_list_entry.port = 443;
+#endif
 	}
 	else {
 		p = &sess->req->uri[4];
 	}
 	if (memcmp(p, "://", 3)) {
+#ifdef WITHOUT_CRYPTLIB
+		set_msg_locked(sess->req, "URI is not http://");
+#else
 		set_msg_locked(sess->req, "URI is not http[s]://");
+#endif
 		assert_pthread_mutex_unlock(&sess->req->mtx);
 		goto error_return;
 	}
@@ -1145,7 +1155,8 @@ ret_return:
 	return ret;
 }
 
-double dmax(double d1, double d2)
+static double
+dmax(double d1, double d2)
 {
 	if (d1 > d2)
 		return d1;
