@@ -29,6 +29,12 @@
 #include "filewrap.h"   /* chsize */
 #include "netwrap.h"
 
+#if defined(_WIN32)
+        #define QSORT_CALLBACK_TYPE __cdecl
+#else
+        #define QSORT_CALLBACK_TYPE
+#endif
+
 /* Maximum length of entire line, includes '\0' */
 #define INI_MAX_LINE_LEN        (INI_MAX_VALUE_LEN * 2)
 #define INI_COMMENT_CHAR        ';'
@@ -41,6 +47,9 @@
 #define INI_INCLUDE_DIRECTIVE   "!include "
 #define INI_INCLUDE_MAX         10000
 
+// Can be exported if needed by someone who wants to poke under the hood.
+static char iniParsedRootValue[1] = {0};
+
 static ini_style_t default_style;
 
 void iniSetDefaultStyle(ini_style_t style)
@@ -289,7 +298,7 @@ static char* read_value(FILE* fp, const char* section, const char* key, char* va
 		if (vp == NULL)
 			break;
 		/* key found */
-		sprintf(value, "%.*s", INI_MAX_VALUE_LEN - 1, vp);
+		strlcpy(value, vp, INI_MAX_VALUE_LEN);
 		return clean_value(value);
 	}
 
@@ -321,7 +330,7 @@ static size_t get_value(str_list_t list, const char* section, const char* key, c
 		if (stricmp(p, key) != 0)
 			continue;
 		if (value != NULL) {
-			sprintf(value, "%.*s", INI_MAX_VALUE_LEN - 1, vp);
+			strlcpy(value, vp, INI_MAX_VALUE_LEN);
 			clean_value(value);
 		}
 		if (vpp != NULL)
@@ -435,9 +444,15 @@ bool iniRemoveSection(str_list_t* list, const char* section)
 {
 	size_t i;
 
-	i = find_section_index(*list, section);
-	if ((*list)[i] == NULL)    /* not found */
-		return false;
+	if (section == ROOT_SECTION) {
+		if (list == NULL || (*list) == NULL || (*list)[0] == NULL || *(*list)[0] == INI_OPEN_SECTION_CHAR)
+			return false;
+		i = 0;
+	} else {
+		i = find_section_index(*list, section);
+		if ((*list)[i] == NULL)    /* not found */
+			return false;
+	}
 	do {
 		strListDelete(list, i);
 	} while ((*list)[i] != NULL && *(*list)[i] != INI_OPEN_SECTION_CHAR);
@@ -449,9 +464,15 @@ bool iniRemoveSectionFast(str_list_t list, const char* section)
 {
 	size_t i;
 
-	i = find_section_index(list, section);
-	if (list[i] == NULL)   /* not found */
-		return false;
+	if (section == ROOT_SECTION) {
+		if (list == NULL || list[0] == NULL || *list[0] == INI_OPEN_SECTION_CHAR)
+			return false;
+		i = 0;
+	} else {
+		i = find_section_index(list, section);
+		if (list[i] == NULL)   /* not found */
+			return false;
+	}
 	do {
 		strListFastDelete(list, i, /* count: */ 1);
 	} while (list[i] != NULL && *list[i] != INI_OPEN_SECTION_CHAR);
@@ -1016,8 +1037,9 @@ char* iniSetIntList(str_list_t* list, const char* section, const char* key
 
 static char* default_value(const char* deflt, char* value)
 {
-	if (deflt != NULL && deflt != value && value != NULL)
-		sprintf(value, "%.*s", INI_MAX_VALUE_LEN - 1, deflt);
+	if (deflt != NULL && deflt != value && value != NULL) {
+		strlcpy(value, deflt, INI_MAX_VALUE_LEN);
+	}
 
 	return (char*)deflt;
 }
@@ -1287,6 +1309,7 @@ static str_list_t ini_read_section_list(FILE* fp, const char* prefix, bool inclu
 	char       str[INI_MAX_LINE_LEN];
 	ulong      items = 0;
 	str_list_t lp;
+	size_t     prefixLen = 0;
 
 	if ((lp = strListInit()) == NULL)
 		return NULL;
@@ -1296,6 +1319,9 @@ static str_list_t ini_read_section_list(FILE* fp, const char* prefix, bool inclu
 
 	rewind(fp);
 
+	if (prefix != NULL)
+		prefixLen = strlen(prefix);
+
 	while (!feof(fp)) {
 		if (fgets(str, sizeof(str), fp) == NULL)
 			break;
@@ -1303,9 +1329,10 @@ static str_list_t ini_read_section_list(FILE* fp, const char* prefix, bool inclu
 			break;
 		if ((p = section_name(str)) == NULL)
 			continue;
-		if (prefix != NULL)
-			if (strnicmp(p, prefix, strlen(prefix)) != 0)
+		if (prefixLen != 0) {
+			if (strnicmp(p, prefix, prefixLen) != 0)
 				continue;
+		}
 		if (!include_dupes && strListFind(lp, p, /* case_sensitive */ false) >= 0)
 			continue;
 		if (strListAppend(&lp, p, items++) == NULL)
@@ -1331,6 +1358,7 @@ static str_list_t ini_get_section_list(str_list_t list, const char* prefix, bool
 	char       str[INI_MAX_LINE_LEN];
 	ulong      i, items = 0;
 	str_list_t lp;
+	size_t     prefixLen = 0;
 
 	if ((lp = strListInit()) == NULL)
 		return NULL;
@@ -1338,15 +1366,19 @@ static str_list_t ini_get_section_list(str_list_t list, const char* prefix, bool
 	if (list == NULL)
 		return lp;
 
+	if (prefix != NULL)
+		prefixLen = strlen(prefix);
+
 	for (i = 0; list[i] != NULL; i++) {
 		SAFECOPY(str, list[i]);
 		if (is_eof(str))
 			break;
 		if ((p = section_name(str)) == NULL)
 			continue;
-		if (prefix != NULL)
-			if (strnicmp(p, prefix, strlen(prefix)) != 0)
+		if (prefixLen != 0) {
+			if (strnicmp(p, prefix, prefixLen) != 0)
 				continue;
+		}
 		if (include_dupes && strListFind(lp, p, /* case_sensitive */ false) >= 0)
 			continue;
 		if (strListAppend(&lp, p, items++) == NULL)
@@ -1371,19 +1403,24 @@ size_t iniGetSectionCount(str_list_t list, const char* prefix)
 	char*  p;
 	char   str[INI_MAX_LINE_LEN];
 	size_t i, items = 0;
+	size_t prefixLen = 0;
 
 	if (list == NULL)
 		return 0;
 
+	if (prefix != NULL)
+		prefixLen = strlen(prefix);
+
 	for (i = 0; list[i] != NULL; i++) {
 		SAFECOPY(str, list[i]);
 		if (is_eof(str))
 			break;
 		if ((p = section_name(str)) == NULL)
 			continue;
-		if (prefix != NULL)
-			if (strnicmp(p, prefix, strlen(prefix)) != 0)
+		if (prefixLen != 0) {
+			if (strnicmp(p, prefix, prefixLen) != 0)
 				continue;
+		}
 		items++;
 	}
 
@@ -1395,12 +1432,16 @@ size_t iniReadSectionCount(FILE* fp, const char* prefix)
 	char* p;
 	char  str[INI_MAX_LINE_LEN];
 	ulong items = 0;
+	size_t prefixLen = 0;
 
 	if (fp == NULL)
 		return 0;
 
 	rewind(fp);
 
+	if (prefix != NULL)
+		prefixLen = strlen(prefix);
+
 	while (!feof(fp)) {
 		if (fgets(str, sizeof(str), fp) == NULL)
 			break;
@@ -1408,9 +1449,10 @@ size_t iniReadSectionCount(FILE* fp, const char* prefix)
 			break;
 		if ((p = section_name(str)) == NULL)
 			continue;
-		if (prefix != NULL)
-			if (strnicmp(p, prefix, strlen(prefix)) != 0)
+		if (prefixLen != 0) {
+			if (strnicmp(p, prefix, prefixLen) != 0)
 				continue;
+		}
 		items++;
 	}
 
@@ -1577,6 +1619,41 @@ iniGetNamedStringList(str_list_t list, const char* section)
 	return lp;
 }
 
+static bool
+addParsedSection(named_str_list_t*** lp, size_t *sections, char *name)
+{
+	named_str_list_t** np = (named_str_list_t**)realloc(*lp, sizeof(named_str_list_t*) * (*sections + 2));
+	if (np == NULL)
+		return false;
+	*lp = np;
+	if (((*lp)[*sections] = (named_str_list_t*)malloc(sizeof(named_str_list_t))) == NULL)
+		return false;
+	if (name == iniParsedRootValue) {
+		(*lp)[*sections]->name = name;
+	}
+	else {
+		if (((*lp)[*sections]->name = strdup(name)) == NULL)
+			return false;
+	}
+	if (((*lp)[*sections]->list = strListInit()) == NULL)
+		return false;
+	*sections += 1;
+	return true;
+}
+
+static bool
+addParsedLine(named_str_list_t** lp, size_t sections, char *data, size_t *keys)
+{
+	if (is_eof(data))
+		return false;
+	if (sections > 0) {
+		SKIP_WHITESPACE(data);
+		if (*data != '\0' && *data != INI_COMMENT_CHAR)
+			strListAnnex(&lp[sections - 1]->list, data, (*keys)++);
+	}
+	return true;
+}
+
 // the 'list' must remain allocated/valid through-out the life of the returned named_str_list
 // as this function does not copy the key=value lines in the original list, it just references them
 named_str_list_t** iniParseSections(const str_list_t list)
@@ -1587,55 +1664,64 @@ named_str_list_t** iniParseSections(const str_list_t list)
 	size_t             sections = 0;
 	size_t             keys = 0;
 	named_str_list_t** lp;
-	named_str_list_t** np;
 
-	if (list == NULL)
-		return NULL;
-
-	// Find first section
-	for (i = 0; list[i] != NULL; ++i) {
-		p = list[i];
-		SKIP_WHITESPACE(p);
-		if (*p == INI_OPEN_SECTION_CHAR)
-			break;
-	}
-	if (list[i] == NULL)
+	if (list == NULL || list[0] == NULL)
 		return NULL;
 
 	if ((lp = (named_str_list_t**)malloc(sizeof(named_str_list_t*))) == NULL)
 		return NULL;
 
-	for (; list[i] != NULL; ++i) {
-		SAFECOPY(str, list[i]);
-		p = section_name(str);
-		if (p != NULL) {
-			if ((np = (named_str_list_t**)realloc(lp, sizeof(named_str_list_t*) * (sections + 2))) == NULL)
-				break;
-			lp = np;
-			if ((lp[sections] = (named_str_list_t*)malloc(sizeof(named_str_list_t))) == NULL)
-				break;
-			if ((lp[sections]->name = strdup(p)) == NULL)
-				break;
-			if ((lp[sections]->list = strListInit()) == NULL)
-				break;
-			++sections;
+	// Find root section if present
+	for (i = 0; list[i] != NULL; ++i) {
+		p = list[i];
+		SKIP_WHITESPACE(p);
+		if (*p)
+			break;
+	}
+
+	if (list[i] != NULL) {
+		// TODO: A comment will create a zero-length root section, which kinda sucks...
+		if (*p != INI_OPEN_SECTION_CHAR) {
+			if (!addParsedSection(&lp, &sections, iniParsedRootValue))
+				goto error_return;
 			keys = 0;
-		} else {
-			p = list[i];
-			if (is_eof(p))
-				break;
-			if (sections > 0) {
+			for (; list[i] != NULL; ++i) {
+				p = list[i];
 				SKIP_WHITESPACE(p);
-				if (*p == '\0' || *p == INI_COMMENT_CHAR)
-					continue;
-				strListAnnex(&lp[sections - 1]->list, p, keys++);
+				if (*p == INI_OPEN_SECTION_CHAR)
+					break;
+				// False return here means it was EOF
+				if (!addParsedLine(lp, sections, list[i], &keys))
+					break;
 			}
 		}
 	}
 
+	for (; list[i] != NULL; ++i) {
+		SAFECOPY(str, list[i]);
+		p = section_name(str);
+		if (p != NULL) {
+			if (!addParsedSection(&lp, &sections, p))
+				goto error_return;
+			keys = 0;
+		} else {
+			// False return here means it was EOF
+			if (!addParsedLine(lp, sections, list[i], &keys))
+				break;
+		}
+	}
+
+	if (sections == 0)
+		goto error_return;
+
 	lp[sections] = NULL;    /* terminate list */
 
 	return lp;
+
+error_return:
+	lp[sections] = NULL;    /* terminate list */
+	iniFreeParsedSections(lp);
+	return NULL;
 }
 
 str_list_t iniGetParsedSectionList(named_str_list_t** list, const char* prefix)
@@ -1644,13 +1730,17 @@ str_list_t iniGetParsedSectionList(named_str_list_t** list, const char* prefix)
 	size_t            count = 0;
 	str_list_t        result = strListInit();
 	named_str_list_t* section;
+	size_t            prefixLen = 0;
+
+	if (prefix != NULL)
+		prefixLen = strlen(prefix);
 
 	for (i = 0; list != NULL && list[i] != NULL; ++i) {
 		section = list[i];
-		if (section->name == NULL)
+		if (section->name == NULL || section->name == iniParsedRootValue)
 			continue;
-		if (prefix != NULL) {
-			if (strnicmp(section->name, prefix, strlen(prefix)) != 0)
+		if (prefixLen != 0) {
+			if (strnicmp(section->name, prefix, prefixLen) != 0)
 				continue;
 		}
 		strListAppend(&result, section->name, count++);
@@ -1663,19 +1753,29 @@ str_list_t iniGetParsedSection(named_str_list_t** list, const char* name, bool c
 	size_t            i;
 	named_str_list_t* section;
 
-	if (name == NULL) // Root section not supported
-		return NULL;
-
 	if (list == NULL)
 		return NULL;
 
 	for (i = 0; list[i] != NULL; ++i) {
+		/*
+		 * We can't declare these below, so can't make them const
+		 * until MSVC supports C99. Just adding braces around the
+		 * const declarations seems to make MSVC crash in random
+		 * places.
+		 * 
+		 * https://gitlab.synchro.net/main/sbbs/-/pipelines/9324
+		 */
+		bool isRootSection;
+		bool isRootMatch;
 		section = list[i];
 		if (section->name == NULL)
 			continue;
-		if (stricmp(section->name, name) == 0) {
+		isRootSection = (section->name == iniParsedRootValue);
+		isRootMatch = isRootSection && (name == NULL);
+		if (isRootMatch || ((name != NULL) && (!isRootSection) && (stricmp(section->name, name) == 0))) {
 			if (cut) {
-				free(section->name);
+				if (!isRootSection)
+					free(section->name);
 				section->name = NULL;
 			}
 			return section->list;
@@ -1692,7 +1792,8 @@ void* iniFreeParsedSections(named_str_list_t** list)
 		return NULL;
 
 	for (i = 0; list[i] != NULL; ++i) {
-		free(list[i]->name);
+		if (list[i]->name != iniParsedRootValue)
+			free(list[i]->name);
 		free(list[i]->list);
 		free(list[i]);
 	}
@@ -2936,6 +3037,415 @@ bool iniWriteFile(FILE* fp, const str_list_t list)
 	return count == strListCount(list);
 }
 
+// Fast parsed INI interface
+struct fp_section {
+	ini_lv_string_t name;	// Unterminated name (borrowed)
+	str_list_t list;	// Allocated at most once (allocated)
+	size_t listLen;		// Number of lines in list
+	size_t originalOrder;	// Original section number
+	bool cut;		// Has been removed (does not free list)
+};
+
+struct fp_list_s {
+	ini_lv_string_t *sectionList;	// Allocated at most once (allocated)
+	size_t firstUncut;	// Where to start/end searching for a section
+	size_t lastUncut;	// Where to start/end searching for a section
+	size_t totalSections;	// The number of sections initially allocated, includes cut sections
+	struct fp_section sections[];
+};
+
+void
+iniFreeFastParse(ini_fp_list_t *s)
+{
+	size_t i;
+
+	if (s == NULL)
+		return;
+	free(s->sectionList);
+	for (i = 0; i < s->totalSections; i++) {
+		// This abuses strListFreeBlock() and assumes it's just a free() wrapper
+		strListFreeBlock((char *)s->sections[i].list);
+	}
+	free(s);
+}
+
+static size_t
+getArraySize(size_t allocSz)
+{
+	return (allocSz - sizeof(ini_fp_list_t)) / sizeof(struct fp_section);
+}
+
+static int QSORT_CALLBACK_TYPE
+iniFastParseCmp(const void *a, const void *b)
+{
+	const struct fp_section *seca = a;
+	const struct fp_section *secb = b;
+	size_t cmpLen;
+	int cmp;
+	bool aLonger = false;
+	bool abSame;
+
+	if (a == NULL) {
+		if (b == NULL)
+			return seca->originalOrder - secb->originalOrder;
+		return -1;
+	}
+	if (b == NULL)
+		return 1;
+	abSame = (seca->name.len == secb->name.len);
+	if (!abSame) {
+		if (seca->name.len > secb->name.len)
+			aLonger = true;
+	}
+	cmpLen = aLonger ? secb->name.len : seca->name.len;
+	cmp = strnicmp(seca->name.str, secb->name.str, cmpLen);
+	if (cmp)
+		return cmp;
+	if (abSame)
+		return seca->originalOrder - secb->originalOrder;
+	if (aLonger)
+		return 1;
+	return -1;
+}
+
+/*
+ * References the original list with whitespace removed, but also has
+ * additional allocation. Allocates 4k (one page) to start, then doubles
+ * on overflow.
+ * 
+ * Returns NULL on error only, allocates 4k even for an empty list.
+ * It is an error to pass a NULL list.
+ * 
+ * If orderedList is true, will create a list of section names in file
+ * order which can be retreived using iniGetFastParsedSectionOrderedList()
+ * That list is not deduped or modified by cut operations.
+ */
+ini_fp_list_t *
+iniFastParseSections(const str_list_t list, bool orderedList)
+{
+	size_t allocSz = 4096;
+	ini_fp_list_t *ret = malloc(allocSz);
+	size_t arraySz = getArraySize(allocSz);
+	size_t i;
+
+	if (!ret)
+		return NULL;
+
+	if (!list) {
+		free(ret);
+		return NULL;
+	}
+
+	ret->firstUncut = 0;
+	ret->totalSections = 0;
+	ret->sectionList = NULL;
+
+	// Root section
+	memset(&ret->sections[0], 0, sizeof(ret->sections[0]));
+	ret->sections[0].list = strListInit();
+
+	if (ret->sections[0].list == NULL)
+		goto error_return;
+
+	for (i = 0; list[i] != NULL; i++) {
+		char *str = list[i];
+		SKIP_WHITESPACE(str);
+		if (*str == INI_OPEN_SECTION_CHAR) {
+			struct fp_section *sect;
+			size_t slen;
+			str++;
+			slen = strlen(str);
+			while (slen && (IS_WHITESPACE(str[slen - 1])))
+				slen--;
+			if (slen && str[slen - 1] == INI_CLOSE_SECTION_CHAR)
+				slen--;
+			else // Discard line
+				continue;
+			ret->totalSections++;
+			if ((ret->totalSections) >= arraySz) {
+				ini_fp_list_t *np;
+				allocSz *= 2;
+				np = realloc(ret, allocSz);
+				if (np == NULL)
+					goto error_return;
+				ret = np;
+				arraySz = getArraySize(allocSz);
+			}
+			sect = &ret->sections[ret->totalSections];
+			sect->list = strListInit();
+			if (sect->list == NULL)
+				goto error_return;
+			sect->listLen = 0;
+			sect->name.str = str;
+			sect->originalOrder = ret->totalSections;
+			sect->cut = false;
+			sect->name.len = slen;
+		}
+		else {
+			if (*str != '\0' && *str != INI_COMMENT_CHAR) {
+				if (!strListAnnex(&ret->sections[ret->totalSections].list, str, ret->sections[ret->totalSections].listLen))
+					goto error_return;
+				ret->sections[ret->totalSections].listLen++;
+			}
+		}
+	}
+	ret->totalSections++;
+
+	if (orderedList) {
+		ret->sectionList = malloc(ret->totalSections * sizeof(*ret->sectionList));
+		if (ret->sectionList == NULL)
+			goto error_return;
+		for (i = 0; i < ret->totalSections; i++) {
+			ret->sectionList[i] = ret->sections[i].name;
+		}
+	}
+	// Sort
+	qsort(ret->sections, ret->totalSections, sizeof(ret->sections[0]), iniFastParseCmp);
+	// Remove duplicates (ugh)
+	for (i = 1; i < ret->totalSections; i++) {
+		int cmp;
+		struct fp_section *seca = &ret->sections[i - 1];
+		struct fp_section *secb = &ret->sections[i];
+		if (seca->name.len != secb->name.len)
+			continue;
+		cmp = strnicmp(seca->name.str, secb->name.str, seca->name.len);
+		if (cmp)
+			continue;
+		if (secb->originalOrder > seca->originalOrder) {
+			ret->totalSections--;
+			if (i < ret->totalSections) {
+				struct fp_section *secc = &ret->sections[i + 1];
+				// This abuses strListFreeBlock() and assumes it's just a free() wrapper
+				strListFreeBlock((char *)secb->list);
+				memmove(secb, secc, sizeof(*secb) * (ret->totalSections - i));
+			}
+		}
+		else {
+			// This abuses strListFreeBlock() and assumes it's just a free() wrapper
+			strListFreeBlock((char *)seca->list);
+			memmove(seca, secb, sizeof(*seca) * (ret->totalSections - i));
+			ret->totalSections--;
+		}
+	}
+	ret->lastUncut = ret->totalSections;
+	if (ret->lastUncut)
+		ret->lastUncut--;
+	return ret;
+
+error_return:
+	free(ret->sections[ret->totalSections].list);
+	ret->sections[ret->totalSections].list = NULL;
+	iniFreeFastParse(ret);
+	return NULL;
+}
+
+struct iniGetFastPrefixStartCmpKey {
+	ini_lv_string_t prefix;
+	struct fp_section *base;
+};
+
+static int QSORT_CALLBACK_TYPE
+iniGetFastPrefixStartCmp(const void *keyPtr, const void *entryPtr)
+{
+	const struct iniGetFastPrefixStartCmpKey *key = keyPtr;
+	const struct fp_section *fp = entryPtr;
+	bool fpIsShorter = fp->name.len < key->prefix.len;
+	size_t cmpLen = fpIsShorter ? fp->name.len : key->prefix.len;
+	int cmp = strnicmp(key->prefix.str, fp->name.str, cmpLen);
+
+	if (cmp)
+		return cmp;
+	if (fpIsShorter)
+		return 1;
+	/*
+	 * We now know the prefix is present... now we need to check if
+	 * the previous entry also has the prefix...
+	 */
+	if (fp == key->base) {
+		// First entry, this is where we start...
+		return 0;
+	}
+	// Move to the previous entry
+	fp--;
+
+	// If the previous entry is shorter, we have the start
+	if (fp->name.len < key->prefix.len)
+		return 0;
+	// Doesn't start with prefix, we're good
+	if (strnicmp(fp->name.str, key->prefix.str, key->prefix.len))
+		return 0;
+	// Does start with prefix, previous is a better start
+	return 1;
+}
+
+static void
+adjustUncuts(ini_fp_list_t *fp)
+{
+	while (fp->sections[fp->firstUncut].cut && fp->firstUncut <= fp->lastUncut)
+		fp->firstUncut++;
+	if (fp->firstUncut > fp->lastUncut)
+		return;
+	while (fp->lastUncut > fp->firstUncut && fp->sections[fp->lastUncut].cut)
+		fp->lastUncut--;
+}
+
+static size_t
+iniGetFastPrefixStart(ini_fp_list_t *fp, const char *prefix)
+{
+	struct iniGetFastPrefixStartCmpKey key = {0};
+	struct fp_section *found;
+	adjustUncuts(fp);
+	if (fp->firstUncut >= fp->totalSections)
+		return SIZE_MAX;
+	if (prefix == NULL || *prefix == 0)
+		return fp->firstUncut;
+	key.prefix.str = prefix;
+	key.prefix.len = strlen(prefix);
+	key.base = &fp->sections[fp->firstUncut];
+
+	found = bsearch(&key, key.base, fp->lastUncut - fp->firstUncut + 1, sizeof(fp->sections[0]), iniGetFastPrefixStartCmp);
+	if (found == NULL)
+		return SIZE_MAX;
+	return found - fp->sections;
+}
+
+ini_lv_string_t **
+iniGetFastParsedSectionList(ini_fp_list_t *fp, const char* prefix, size_t *sz)
+{
+	size_t i;
+	size_t cnt = 0;
+	size_t prefixLen = 0;
+	ini_lv_string_t **ret;
+	if (fp == NULL) {
+		if (sz)
+			*sz = 0;
+		return NULL;
+	}
+	ret = malloc(sizeof(ini_lv_string_t *) * (fp->lastUncut - fp->firstUncut + 1));
+	if (ret == NULL) {
+		if (sz)
+			*sz = 0;
+		return ret;
+	}
+	if (prefix)
+		prefixLen = strlen(prefix);
+	i = iniGetFastPrefixStart(fp, prefix);
+	if (i != SIZE_MAX) {
+		for (; i <= fp->lastUncut; i++) {
+			if (fp->sections[i].name.str == NULL)
+				continue;
+			if (fp->sections[i].cut)
+				continue;
+			if (fp->sections[i].name.len < prefixLen)
+				break;
+			if (prefixLen) {
+				if (strnicmp(fp->sections[i].name.str, prefix, prefixLen))
+					break;
+			}
+			ret[cnt] = &(fp->sections[i].name);
+			cnt++;
+		}
+	}
+	if (sz)
+		*sz = cnt;
+	return ret;
+}
+
+static int QSORT_CALLBACK_TYPE
+iniGetFastParsedSectionCmp(const void *keyPtr, const void *entPtr)
+{
+	const struct fp_section *fp = entPtr;
+	const ini_lv_string_t *name = keyPtr;
+	size_t cmplen;
+	bool entShorter;
+	int cmp;
+
+	if (fp->name.str == NULL) {
+		if (name == NULL || name->str == NULL)
+			return 0;
+	}
+	if (name == NULL || name->str == NULL)
+		return -1;
+	entShorter = fp->name.len < name->len;
+	cmplen = entShorter ? fp->name.len : name->len;
+	if (cmplen) {
+		// The assumption here is that if fp->name.str == NULL, cmplen will be zero
+		// coverity[FORWARD_NULL:SUPPRESS]
+		cmp = strnicmp(name->str, fp->name.str, cmplen);
+	}
+	else
+		cmp = 0;
+	if (cmp == 0) {
+		if (fp->name.len == name->len)
+			return 0;
+		if (entShorter)
+			return 1;
+		return -1;
+	}
+	return cmp;
+}
+
+static str_list_t
+iniHandleFoundSection(struct fp_section *found, ini_fp_list_t *fp, bool cut)
+{
+	if (found == NULL)
+		return NULL;
+	if (found->cut)
+		return NULL;
+	if (cut) {
+		found->cut = true;
+		if (found == &fp->sections[fp->firstUncut])
+			fp->firstUncut++;
+		if (found == &fp->sections[fp->lastUncut] && fp->lastUncut)
+			fp->lastUncut--;
+	}
+	return found->list;
+}
+
+str_list_t
+iniGetFastParsedSection(ini_fp_list_t *fp, const char* name, bool cut)
+{
+	ini_lv_string_t nameLV;
+	struct fp_section *found;
+
+	nameLV.str = name;
+	nameLV.len = name ? strlen(name) : 0;
+	if (fp == NULL)
+		return NULL;
+	adjustUncuts(fp);
+	if (fp->firstUncut > fp->lastUncut)
+		return NULL;
+
+	found = bsearch(&nameLV, &fp->sections[fp->firstUncut], fp->lastUncut - fp->firstUncut + 1, sizeof(fp->sections[0]), iniGetFastParsedSectionCmp);
+	return iniHandleFoundSection(found, fp, cut);
+}
+
+str_list_t
+iniGetFastParsedSectionLV(ini_fp_list_t *fp, ini_lv_string_t* name, bool cut)
+{
+	struct fp_section *found;
+	if (fp == NULL)
+		return NULL;
+	adjustUncuts(fp);
+	if (fp->firstUncut > fp->lastUncut)
+		return NULL;
+
+	found = bsearch(name, &fp->sections[fp->firstUncut], fp->lastUncut - fp->firstUncut + 1, sizeof(fp->sections[0]), iniGetFastParsedSectionCmp);
+	return iniHandleFoundSection(found, fp, cut);
+}
+
+ini_lv_string_t *
+iniGetFastParsedSectionOrderedList(ini_fp_list_t *fp)
+{
+	return fp->sectionList;
+}
+
+void
+iniFastParsedSectionListFree(ini_lv_string_t **list)
+{
+	free(list);
+}
+
 #ifdef INI_FILE_TEST
 void main(int argc, char** argv)
 {
