@@ -45,6 +45,13 @@ typedef struct {
 	char*	literal_separator;
 } ini_style_t;
 
+typedef struct {
+	const char *str;
+	size_t len;
+} ini_lv_string_t;
+
+typedef struct fp_list_s ini_fp_list_t;
+
 #if defined(__cplusplus)
 extern "C" {
 #endif
@@ -212,6 +219,7 @@ DLLEXPORT str_list_t 	iniGetSection(str_list_t, const char *section);
 DLLEXPORT str_list_t 	iniCutSection(str_list_t, const char *section);
 
 #if !defined(NO_SOCKET_SUPPORT)
+// These functions return/accept IP addresses in *network* byte order
 DLLEXPORT uint32_t 		iniReadIpAddress(FILE*, const char* section, const char* key
 							,uint32_t deflt);
 DLLEXPORT uint32_t 		iniGetIpAddress(str_list_t, const char* section, const char* key
@@ -316,6 +324,15 @@ DLLEXPORT str_list_t	iniGetParsedSection(named_str_list_t**, const char* section
 DLLEXPORT str_list_t 	iniGetParsedSectionList(named_str_list_t**, const char* prefix);
 DLLEXPORT void*			iniFreeParsedSections(named_str_list_t** list);
 
+/* Fast functions */
+DLLEXPORT ini_fp_list_t * iniFastParseSections(const str_list_t list, bool orderedList);
+DLLEXPORT ini_lv_string_t **iniGetFastParsedSectionList(ini_fp_list_t *fp, const char* prefix, size_t *sz);
+DLLEXPORT str_list_t iniGetFastParsedSection(ini_fp_list_t *fp, const char* name, bool cut);
+DLLEXPORT str_list_t iniGetFastParsedSectionLV(ini_fp_list_t *fp, ini_lv_string_t* name, bool cut);
+DLLEXPORT void iniFastParsedSectionListFree(ini_lv_string_t **list);
+DLLEXPORT void iniFreeFastParse(ini_fp_list_t *s);
+DLLEXPORT ini_lv_string_t *iniGetFastParsedSectionOrderedList(ini_fp_list_t *fp);
+
 /*
  * Too handy to leave internal
  */
